#!/bin/bash
# htaccess2index.bash
# 10/22/2020 William Craig - bill@wrcraig.com  https://wrcraig.com

# This Linux bash script is called by Run-Htaccess2index-Recursively.bash to run an update on index.html and index2.html in all of the subdirectories of the base directory specified in htaccess-config.bash. This script reads the directory specified and each of it's subdirectories, extracting the filenames (and the play-length of any A/V files therein). It also reads an apache2 style FancyIndex .htaccess file, or a similarly formatted file, containing descriptive info about the filename (a LibreOffice spreadsheet named enterdescriptions.ods is included to make data entry easier, and which can output a text file in the proper format).

# It then produces html download links to the files and combines the Description data from .htaccess (or similar) with the user's headers and footers into files named "index.html" and "index2.html", the first sorted by filename and the second sorted by description.

# DO NOT RUN AS ROOT

#  See the README file (and below) for details.

# REQUIRES "mediainfo" to determine the play-length of audio and video files (sudo apt install mediainfo). If play length is not required then do not install mediainfo. Processing time will be reduced by up to a factor of 10.

# REQUIRES that there are no spaces in the filenames or directories. You may use the accompanying file (ReplSpacesinSubdirs.Filenames.bash) to recursively replace any spaces in the source and sub directories and all filenames with dots. At the command line enter "bash ReplSpacesinSubdirs.Filenames.bash"




######## TO RUN THIS SCRIPT RECURSIVELY (through all subdirectories below the directory of the calling script, "Run-Htaccess2index-Recursively.bash"

# 1. Edit the script, "htaccess-config.bash", to set the parameters for this script. The calling script will run this script in the user's specified base directory. Index.html and index2.html will be created in every subdirectory. Run the script at the command line (bash <path>/Run-Htaccess2index-Recursively.bash). 

#   There is an option in "htaccess-config.bash" to additionally run this script in the specified base directory or non-recursively.

# 2. Be sure to differentiate directory names from partial filenames. You can do this by adding a tag to the end of directory names. You may use ChangeDirectoryNames.sh to make mass changes. Then enter "sh <path>/ChangeDirectoryNames.sh" at the command line.

# 3. Sample of a line from description.txt or .htaccess:    AddDescription "<b>2014, IMDB 6.8, </b>An exploration of the last quarter century of the great, if eccentric, British painter J.M.W. Turner's life.<hr>" Mr.Turner.mkv      * Note that since double quotes are required around the file description, any additional double quotes within the description may cause very unexpected results.

# 4. A LibreOffice spreadsheet named enterdescriptions.ods is included to make data entry easier, and which can output a text or a comma separated variable file in the proper format using the save-as dialog. Be sure to blank out any delimiters if given that option. Remember, after saving the spreadsheet as a text file the spreadsheet will display the text file. You will have to close it without saving it and reopen the spreadsheet.




########################################################
##################### Main Script ######################
### If you change this you better have a backup copy ###
########################################################
# set up display when calling default browser 
export DISPLAY=:0

# Assumes that all the scripts are in the same directory,
# change to the scripts directory to access the config file
MY_PATH="`dirname \"$0\"`"              # relative
MY_PATH="`( cd \"$MY_PATH\" && pwd )`"  # absolutized and normalized
if [ -z "$MY_PATH" ] ; then
  echo "for some reason, the path is not accessible to the script."
  
  # Send error message directly to default browser because Logfile is not defined yet
  
  echo '<p></p><center><font style="background-color: #00FF00"> ERROR - Webmaster, check Logfile, the path to the scripts is not accessible. </font></center><p></p>' >> errorfile.html
  xdg-open errorfile.html
  sleep 1
  printf .
  sleep 1
  printf .
  sleep 1
  printf .
  sleep 1
  printf .
  echo
  rm errorfile.html
  exit 1
fi

scriptpath="$MY_PATH"

if [ -f $scriptpath/htaccess-config.bash ]
then
# Import functions and variables from the config and functions files
source $scriptpath"/htaccess-config.bash"
source $scriptpath"/functions.bash"
else
  echo "All scripts should be in the same directory."
    
  # Send error message directly to browser because Logfile may not be defined yet 
  echo '<p></p><center><font style="background-color: #00FF00"> ERROR - Webmaster, all scripts should be in the same directory. </font></center><p></p>' >> errorfile.html
  xdg-open errorfile.html
  sleep 1
  printf .
  sleep 1
  printf .
  sleep 1
  printf .
  sleep 1
  printf .
  echo
  rm errorfile.html
    exit 1
fi


################################################
###### Looks ok, begin processing
################################################

# initialize variables to flag errors
Logflag=0
NDflag=0

      # ENTER A LABEL FOR THE KIND OF FILES BEING INDEXED (using directory name)
kind=${PWD}
kind=${kind##*/}    # retain the part after the last /

     # reduce the current directory name
dir=${PWD}
dir=${dir##*/}    # retain the part after the last /

     # DEFINE THE PATH TO FILES TO BE READ AND INDEXED. The newly created index files will be placed here. MUST BE SET TO CURRENT DIRECTORY (${PWD}) IF RECURSION IS TO BE USED ON SUBDIRECTORIES (Run-Htaccess2index-Recursively.bash) #####

PathToFiles=${PWD}

# Enforce not run as root
if [[ $EUID -eq 0 ]]; then
  echo '<br><font style="background-color: #00FF00">>></font> You must NOT run this script as Root.' >> $Logfile
  highlight "Cannot run as Root."
  if [ "$RunInBaseDirectory" = y ] && [ "$BaseDirectory" = "${PWD}" ]
  then
    # Send error message directly to browser because base directory may have a static webpage we don't want to change
    echo '<p></p><center><font style="background-color: #00FF00"> ERROR - Webmaster, You must NOT run this script as Root. </font></center><p></p>' >> errorfile.html
    xdg-open errorfile.html
    sleep 1
    printf .
    sleep 1
    printf .
    sleep 1
    printf .
    sleep 1
    printf .
    echo
    rm errorfile.html
  else
    if [ -f index.html ]
    then
      sed -i '12i <p></p><center><font style="background-color: #00FF00"> ERROR - Webmaster, check Logfile </font></center><p></p>' index.html
    fi  
  fi
  exit 1
fi

# Enforce not run in root directory
if [ ${PWD} = / ]
  then
    highlight "You must not run this script on the root directory!!!!!"
    echo '<br><font style="background-color: #00FF00">>></font> You must not run this script on the root directory!!!!!' >> $Logfile
    if [ "$RunInBaseDirectory" = y ] && [ "$BaseDirectory" = "${PWD}" ]
    then
    # Send error message directly to default browser because base directory may have a static webpage we don't want to change
    echo '<p></p><center><font style="background-color: #00FF00"> ERROR - Webmaster, You must NOT run this script in the root directory. </font></center><p></p>' >> errorfile.html
    xdg-open errorfile.html
    sleep 1
    printf .
    sleep 1
    printf .
    sleep 1
    printf .
    sleep 1
    printf .
    echo
    rm errorfile.html
  else
    if [ -f index.html ]
    then
      sed -i '12i <p></p><center><font style="background-color: #00FF00"> ERROR - Webmaster, check Logfile </font></center><p></p>' index.html
    fi
    echo
  fi
  exit 1
fi

# Check if we can write to current directory
touch testfi.log
if [ $? -ne 0 ]
then
  highlight "Cannot write to current directory ("$PathToFiles"). Check directory permissions. "$dir
    if [ -f $errorsound ]
    then
        echo aplay -q $errorsound && play -q $errorsound -t alsa
    fi
  echo '<br><font style="background-color: #00FF00">>></font> Cannot write to current directory ('${PathToFiles}'). Check directory permissions.</font>' >> $Logfile
    
  if [ "$RunInBaseDirectory" = y ] && [ "$BaseDirectory" = "${PWD}" ]
  then
    # Send error message directly to default browser because base directory may have a static webpage - and we can't write to it anyway!
    echo '<p></p><center><font style="background-color: #00FF00"> ERROR - Webmaster, Cannot write to the current directory ('${PathToFiles}'). Check directory permissions. </font></center><p></p>' >> errorfile.html
    xdg-open errorfile.html
    sleep 1
    printf .
    sleep 1
    printf .
    sleep 1
    printf .
    sleep 1
    printf .
    echo
    rm errorfile.html
  else
    if [ -f index.html ]
    then
    # Put notice at top of index.html and exit
    sed -i '12i <p></p><center><font style="background-color: #00FF00"> ERROR - Webmaster, check Logfile </font></center><p></p>' index.html
    fi
  fi
  exit 1
else
 if [ -e testfi.log ]
   then
   chmod --preserve-root 666 testfi.log
   rm testfi.log
  fi
fi

# Here we set the path and name of the file to be output in filename order in the current directory. The filename should be "index.html" or browsers will ignore it and use the ".htaccess" file to list files. if ".htaccess" is not set up and "index.html" is not present, the browser has free reign to look through your entire file system!

# define temporary name of index file (Alfa Order), to be renamed  to index.html later
      OutputFile=tempindex.html

# define name of temporary index file, to be deleted later
      placeholderindex=index.html

# Here we set the path and temporary name of the file to be output in description order. Here the filename can be "(anything).html"
      OutputFile2=tempindex2.html

# define name of temporary index file, to be deleted later
      placeholderindex2=index2.html

#  use a function to find the path to the logfile to create NoDescription.log in the same directory
      locateNDlog

# Define variable for two different "times through" for alphabetical list and for description order list.
# initialize timesthrough for # of times through the loop: 1= both Name & Description order indexes, 2= Description index only
      timesthrough=1
      
# Initialize counter to insert a gototop link periodically
      counter=1

# check if we can write to home directory for flag files to carryover info through recursions
flagdir=$HOME
touch ${flagdir}/testfi.log

if [ -e ${flagdir}/testfi.log ]
then
  rm ${flagdir}/testfi.log
else
  highlight "Cannot write to "$flagdir" Check directory permissions. Access needed to create temporary files for errors "
    if [ -f $errorsound ]
    then
        echo aplay -q $errorsound && play -q $errorsound -t alsa
    fi
  echo '<br><font style="background-color: #00FF00">>></font> Cannot write to '$flagdir.'</br> Check directory permissions. Access needed to create temporary files for errors.</font>' >> $Logfile
  if [ -f index.html ]
    then
      # Put notice at top of index.html and exit
      sed -i '12i <p></p><center><font style="background-color: #00FF00"> ERROR FOUND - Webmaster, check Logfile </font></center><p></p>' index.html
  fi
  exit 1
fi


# Check for valid timesthrough
if [ "${timesthrough}" -gt "2" ] || [ "${timesthrough}" -lt "1" ]
then
	highlight "Timesthrough must be either 1 or 2, Line ~253"
    if [ -f $errorsound ]
    then
        echo aplay -q $errorsound  && play -q $errorsound -t alsa
    fi
echo '<br><font style="background-color: #00FF00">>></font> Line ~253 Timesthrough must be either 1 or 2 ' >> $Logfile
if [ -f index.html ]
then
  sed -i '12i <p></p><center><font style="background-color: #00FF00"> ERROR FOUND - Webmaster, check Logfile </font></center><p></p>' index.html
fi
exit 1
fi

# If error flag files were set in a prior run commit them to memory variables and delete the flag files

if [ -f ${flagdir}/.flagerrorfile ]
then
  Logflag=1
  rm ${flagdir}/.flagerrorfile
fi

if [ -f ${flagdir}/$.NDflagfile ]
then
  NDflag=1
  rm ${flagdir}/$.NDflagfile
fi

# Notify user that we are now overwriting their current index.html
if [ -f ${placeholderindex} ]
then
echo
highlight "Overwriting the current" ${placeholderindex} "file for ${kind} and replacing it with one based on the file containing descriptions of ${kind}."
echo
fi


# Fill up any empty strings in the user's excluded file names
if [ -z ${file1} ]
then
file1=notemptystring
fi

if [ -z ${file2} ]
then
file2=notemptystring
fi

if [ -z ${file3} ]
then
file3=notemptystring
fi

if [ -z ${file4} ]
then
file4=notemptystring
fi

if [ -z ${file5} ]
then
file5=notemptystring
fi

if [ -z ${file6} ]
then
file6=notemptystring
fi

if [ -z ${file7} ]
then
file7=notemptystring
fi

if [ -z ${file8} ]
then
file8=notemptystring
fi

if [ -z ${file9} ]
then
file9=notemptystring
fi

if [ -z ${file10} ]
then
file10=notemptystring
fi

	#######################
	# Start main DO loop  #
	#######################
	
until [ "${timesthrough}" -gt "2" ]
do

#  Define variables for alphabetical list for first time through
if [ "${timesthrough}" -eq "1" ]
then
type="Filename"
type1="Description"
out=${OutputFile##*/}
out1=${OutputFile2##*/}
pout=${placeholderindex##*/}
pout1=${placeholderindex2##*/}
fi

# Define variables for description-ordered list for second time through
if [ "${timesthrough}" -eq "2" ]
then
type="Description"
type1="Filename"
out=${OutputFile2##*/}
out1=${OutputFile##*/}
pout=${placeholderindex2##*/}
pout1=${placeholderindex##*/}
fi



# Make the placeholder and the temporary index files
Makehtml



#################### Process the filename & A/V Duration #####################

# Set color of length of a/v files in index html files
durationcolor=firebrick

# Initialize variables
Duration=""
l=""
# set variable to sense errors in each directory and consolidate errors so we post only one error message on that web page
errorfound=0

# Loop through all files in the path given in the config file (default sort is alphabetical order)
for file in *
do
f=$file

echo
echo "Processing" $f "for" $kind "in" $type "order."    # show user the current filename being processed

# delete if we find a zero length file
if [ ${#f} -le 2 ]
then
  echo "Removing zero length file" $file "in" $dir
  echo "<br><font style='background-color: #00FF00'>>></font> Removing zero length file' $file 'in' $dir '</font>" >> $Logfile
  rm $f
fi

# check for spaces in filename
if [[ "$f" == *" "* ]]     
then
  currentdir=${PWD}"/"
  highlight "Spaces found in filename. Skipping" $currentdir$file"."
  
  if [ "${timesthrough}" -eq "1" ]
  then
    echo '<br><font style="background-color: #00FF00">>></font> Spaces found in filename, cannot process. Skipping' $currentdir$file'. </font>' >> $Logfile
  fi
  
  #  Set flag that we have an error so later we notify user to check the log
  Logflag=1
  continue
fi

	   # Initialize subtitles variable
subtitles=""

##### skip files with excluded extensions
 ext=${file##*.}             # strip filename down to only the extension
 ext=${ext^^}                # Capitalize it
 
# Check file extension against the excluded list (Case statement works since each file has only one extension and we are checking one file at a time).
case "$ext" in
   "${ext1^^}")
      echo "  Excluding extension ."$ext1", go to next file"
      continue
	  ;;
   "${ext2^^}")
      echo "  Excluding extension ."$ext2", go to next file"
      continue
	  ;;
   "${ext3^^}")
      echo "  Excluding extension ."$ext3", go to next file"
      break
	  ;;
   "${ext4^^}")
      echo "  Excluding extension ."$ext4", go to next file"
      continue
	  ;;
   "${ext5^^}")
      echo "  Excluding extension ."$ext5", go to next file"
      continue
	  ;;
   "${ext6^^}")
      echo "  Excluding extension ."$ext6", go to next file"
      continue
	  ;;
   "${ext7^^}")
      echo "  Excluding extension ."$ext7", go to next file"
      continue
	  ;;
   "${ext8^^}")
      echo "  Excluding extension ."$ext8", go to next file"
      continue
	  ;;
   "${ext9^^}")
      echo "  Excluding extension ."$ext9", go to next file"
      continue
	  ;;
   "${ext10^^}")
      echo "  Excluding extension ."$ext10", go to next file"
      continue
	  ;;
              # if this is a subtitle file save special instructions for it
    "SRT") subtitles='<br/><b><font style="color: green">Subtitle file, keep in the same directory as the video file. Should be the same name as the video file but with a .srt extension.</font></b>'
          ;;
#        *) (what to do when all the other cases fail);;
esac


# Check filename against the excluded list adding wildcards before and after the filename
case ${f} in
   *$file1*    )
      echo "  Excluding file "$file1". Go to next file"
      continue
	  ;;
   *$file2*    )
      echo "  Excluding file "$file2". Go to next file"
      continue
	  ;;
   *$file3*    )
      echo "  Excluding file "$file3". Go to next file"
      continue
	  ;;
   *$file4*    )
      echo "  Excluding file "$file4". Go to next file"
      continue
	  ;;
   *$file5*    )
      echo "  Excluding file "$file5". Go to next file"
      continue
	  ;;
   *$file6*    )
      echo "  Excluding file "$ile6". Go to next file"
      continue
	  ;;
   *$file7*    )
      echo "  Excluding file "$file7". Go to next file"
      continue
	  ;;
   *$file8*     )
      echo "  Excluding file "$file8". Go to next file"
      continue
	  ;;
   *$file9*    )
      echo "  Excluding file "$file9". Go to next file"
      continue
	  ;;
   *$file10*   )
      echo "  Excluding file "$file10". Go to next file"
      continue
	  ;;
#          *)        (what to do when all the other cases fail);;
esac




if [ "$mediainf" = "t"  ]
then
  
# Find the duration of media files using path and full file name
sizefinal=""
# If the file is a Directory
if [ -d "$file" ]
then
  echo $f "is a Directory"
  Duration="<b> -- More files inside --</b>"
  len=""
  # ensure we can all read, write, and execute the directory
  chmod --preserve-root 777 $file
else
# get the size of the file
   size=$(wc -c <"$file")
   # reduce to kb
   size=$(( $size/1024/1024 ))
   # get first digit
   size1=$(( $size/1000 ))
   # get last 3 digits
   endsize=${size: -3}
   # combine to show total megabytes
   sizefinal="("$size1"."$endsize"M)"


# try to get a video file duration (length)
if [ $mediainf = "t" ]
then
  Duration=$(mediainfo --Inform="Video;%Duration/String3%" $file) > /dev/null 2>&1
  if [ ${#Duration} -gt 0 ]
  then
     len="Length: "
     echo "video file"   
  else
     # Try to get an audio file duration
     Duration=$(mediainfo --Inform="Audio;%Duration/String3%" $file) > /dev/null 2>&1
  if [ ${#Duration} -gt 0 ]
    then
       len="Length: "
       echo "audio file"
     else
       # Is neither audio or video file
       len=""
    fi
  fi   
fi
fi
fi


# Trim Duration to remove nanoseconds
   Duration=${Duration%.*}


# Format output for all A/V files with temp filename,html link, filename, and duration of any A/V files, hold in memory for later use as the first line in index.html

   Output='<hr><b><a href='$f'>'$f'</a><font style="color: '$durationcolor'"> '$len$Duration'</font> '$sizefinal'</b>'$subtitles'<br>'
   Outputb=$len 
   Outputc=$Duration'</font> '$sizefinal'</b>'$subtitles'<br>'
   # put all 3 together
  # Output=$Outputa$Outputb$Outputc



################# Find the Description of the File ##################

echo "Getting description..."

# Search for the file defined in the config file as DescriptionFile in the current directory. If not there, try higher directories until finding the DescriptionFile or reaching the base directory. If found, extract description of the file as Output1

# Set some variables
mydir=$PWD
Output1="nothing"

# Start searching

while [ "$Output1" = "nothing" ]
  do
    currentdir=${PWD}    
    if [ ! -f "$DescriptionFile" ]
    then
        # did not find $DescriptionFile 
        Output1=nothing
    	cd ..
        # Are we in the base directory?
        if [ "$currentdir" = "$BaseDirectory" ]
        then
     	    # ended up in base directory, next file
            Output1=""
            echo "Cannot find" $DescriptionFile >> $Logfile
            if [ -f index.html ]
            then
	        sed -i '12i <p></p><center><font style="background-color: #00FF00"> ERROR FOUND - Webmaster, check Logfile. Cannot find DescriptionFile </font></center><p></p>' index.html	    
	    fi
            highlight "599 Cannot find" $DescriptionFile.
	        exit 1
	    fi
        # We are not in the base directory, MOVING UP A DIR from $currentdir
    	continue 
    else
        # Found the DescriptionFile
    	# Look for the first occurrence of the file description
        Output1=$(grep -m 1 "$file" "$DescriptionFile")
       	if [ "$Output1" = "" ]
	    then 
    	    # Failed to find description for $f, go to next file
   	        continue 
   	    else
	        if [ "${timesthrough}" -eq "1" ]
            then
	           # See if there is more than 1 occurrence of the description for this filename
	           duplicate=$(grep -c "${f}" "$DescriptionFile")
	           if [ "$duplicate" -gt 1 ]
	           then
        	      echo "There are" $duplicate "matching lines in" $DescriptionFile "for" $file 
	              echo '<br><font style="background-color: #00FF00">>></font> There are' $duplicate 'matching lines in' $DescriptionFile 'for' $file '</font>' >> $Logfile
	    	      Logflag=1
	           fi
	        fi
   	    fi
    fi
done

cd $mydir

# Cutting apart the string from Output1 using bash/cut/split

# Practical Rules using 6 as the marker to cut on:
#  a=${MYVAR%6*}     # retain the part before the last 6
#  b=${MYVAR%%6*}    # retain the part before the first 6
#  c=${MYVAR#*6}     # retain the part after the first 6
#  d=${MYVAR##*6}    # retain the part after the last 6
#  middle=${c%%6*}   # retain the part after the first 6, and within that, retain the part before the remaining 6. Since there are only 2 "6", you get the same answer below with only 1 % mark
#  middle2=${c%6*}

#example:
#echo a= ${a}
#echo b= ${b}
#echo c= ${c}
#echo d= ${d}
#echo ends= $b $d
#echo middle=${middle}
#echo middle2=${middle2}
#echo middle=${middle} 6 $d  #add anything you want to the string

# Misplacing the * usually results in no cuts to the string

    # Sample of line being cut: AddDescription "<b>2014, IMDB 6.8, Drama </b>An exploration of the last quarter century.<hr>" Mr.Turner.mkv

    # Sample of directory listing line: AddDescription "<b>2020, IMDB 8.6, 6 Seasons. History </b>Vikings the ocean.<hr>" Vikings=

  MYVAR=$Output1

  WholeLessTitle=${MYVAR%<hr>*}  
  preamble=${MYVAR%%<b*}   

  # Check for a bad preamble

  adddescr='AddDescription "'

  if [ "$preamble" != "$adddescr" ] && [ "$preamble" != "" ]
    then
	Logflag=1
    	if [ "${timesthrough}" = "1" ]
    	then
      	    highlight "There is a bad preamble in:" $dir/$f
      	    echo $Output1
      	    echo "See" $Logfile "for details."
      	    errorfound=1
      	    echo '<br><font style="background-color: #00FF00">>></font> There is a bad preamble for </font><font style="color: Purple"><b> '$dir/$f'.</b></font><br/> This may affect the working of the script, <font style="color: Red"><b>and may misconfigure your web server until fixed. </b></font>It may also affect the index file. The preamble reads as follows: ['$preamble']. It should read: ['$adddescr'].' >> $Logfile
        fi
  fi
	
      # Cut the line apart from .htaccess pertaining to the file to reshuffle it for the index file
  title=${MYVAR##* }  # retain the part after the last space, the filename (Mr.Turner.mkv)
# remove the title then retain the part after the first < leaving just the description text
  middle=${WholeLessTitle#*<}  # (<b>2014, IMDB 6.8, Drama </b>An exploration of the last quarter century.)
  thruinfo=${middle%%</b>*}  # retain the part before the first </b> (b>2014, IMDB 6.8, Drama )
  middle=${middle##*/b>}  # retain the part after the last /b>  (An exploration of the last quarter century.)
  missing1="<"      # Restore part lost in the cut
  missing2="</b>"   # Restore part lost in the cut


# The pieces not used are:
# $preamble   =   AddDescription "
# $title      =   Vikings=

# The pieces to be used are:
# ${missing1} =  <
# ${thruinfo} =  b>2020, IMDB 8.6, 6 Seasons. History
# ${missing2} =  </b>
# ${middle}   =  Vikings the ocean.<hr>

# initialize color for thruinfo
  color="<font style='color: deeppink'>"
  endcolor="</font>"

# See if we failed to find a description to match the filename (title is less than two characters long)
if [ ${#title} -lt 2 ]
then
  missing1=""
  missing2=""
  if [ "${timesthrough}" = "1" ]
  then
    NDflag=1
    echo
    echo "    Cannot find a Description. Check for a case mismatch or an extra space in the filename. Could also be curly quotes instead of straight quotes. Sending filename to the list at" $NoDescription
    echo "Cannot find a Description for" $f ". Check for a case mismatch or an extra space in the filename. Could also be using curly quotes instead of straight quotes." >> $NoDescription
    chmod --preserve-root 766 $NoDescription
  fi
  else
  echo "   Success!"
fi

# Check for extra unescaped quotes in the description
char1='\"'
char2='\'
quote=$(echo "${Output1}" | awk -F"${char1}" '{print NF}')
escape=$(echo "${Output1}" | awk -F"${char2}" '{print NF}')
difference=$(( $quote - $escape ))

if [ "${timesthrough}" = "1" ]
then
  if [ $difference -ne 0 ] && [ $difference -ne 2 ]  #(&& is the "and" operator, "||" is the or operator)
  then
    Logflag=1
    highlight "There should be double quotes around, but not within the description, otherwise things can be really messed up."
    echo "Remove any extra internal double quotes in the description of" $dir/$f"."
    echo "Current description now reads:" $missing1$thruinfo$missing2$middle
    echo '<br><font style="background-color: #00FF00">>></font> There should be double quotes around, but not within the description, otherwise things can get really messed up. Remove internal quotes in the description for:</font><b><font style="color: Purple">' $dir/$f '</font></b><br/>Current description now reads:' $missing1$thruinfo$missing2$middle >> $Logfile
  fi
fi

# Put description pieces and color all together
  OutputM=${color}${missing1}${thruinfo}${endcolor}${missing2}${middle}
  
let "counter++"

############# if 1st time through #################
if [ "${timesthrough}" -eq "1" ]
then
  # Add the description to the name and duration line and send to index file by name
  outemp="$Output $OutputM"
  echo $outemp >> $out
  
  if [ "${#outemp}" -lt "50" ]
  then
    highlight "1st time thru, It appears that" $outemp "is < 50, not created properly                        "
    Logflag=1
    echo '<br><font style="background-color: #00FF00">>></font> It appears that' $outemp 'was not created properly</font>' >> $Logfile
  fi
  if [ "${counter}" -gt "6" ]
  then
    echo "<p></p>" >> $out
    echo "<hr><font style="color: red"><b><a href="#top">--- Go to Top ----</a></b></font>" >> $out
    counter=1
  fi
fi

############# if 2nd time through #################
# Add the name and duration to the description & send to temp file for later sorting
if [ "${timesthrough}" -eq "2" ]
then
  marker="^"         # initialize a carat(^) as a marker for cutting the string
  outemp="${OutputM} $marker ${Output}"     # Insert the marker and send to the sort file
  echo ${outemp} >> tobesorted
fi

        # keep looping until all files are processed
done
######### 2nd time      # Once all files are processed, sort by description
if [ "${timesthrough}" -eq "2" ]
then
  if [ -f tobesorted ]
  then
    sort -r tobesorted > sorted
   
        # Split line to put in final order
 # Sample of line being cut: <b>2014, IMDB 6.8, </b>An exploration of the last quarter century of the great, if eccentric, British painter J.M.W. Turners life. ^ <hr><b><a href=\"${file}\">${file##*/}</a><font style="color: green"> -Running Time:</font></b> ${Duration}<br>

# Read each line of sorted, split it & send it to out
  DONE=false
  until $DONE ;do
    read || DONE=true

    VAR=$REPLY
    desc=${VAR%\^*}  # retain the part before the last ^
    
    name=${VAR##*^}  # retain the part after the last ^
    b=${VAR#*$PathToFiles}    # retain the part after the first path to videos
    fn=${b%%\>*}     # retain the part before the first >
    # Send 2 parts to final index2.html
    echo "${name} ${desc}" >> $out

    # Insert a "Go to Top" link every few entries
    if [ "${counter}" -gt 6 ]
    then
      echo "<p></p>" >> $out
      echo "<hr><font style="color: red"><b>-------- <a href="#top">Go to Top</a> --------</b></font>" >> $out
      counter=1
    fi
    let "counter++"
  done < sorted
  let "counter++"
  fi
fi
 
# Write footer info to output file
echo $footer1 >> $out
echo $footer2 >> $out
echo $footer3 >> $out
echo $footer4 >> $out
echo $footer5 >> $out
echo $footer6 >> $out

# ensure we can all read and write the file
chmod --preserve-root 766 $pout


# Cleanup

if [ -e tobesorted ]
then
  rm tobesorted
fi

if [ -e sorted ]
then
 rm sorted
fi

# overwrite temp index with new index file
if [ -f $out ]
then
  chmod --preserve-root 766 $out
  mv $out $pout
  if [ $? -ne 0 ]; then
      highlight "Something went wrong with the mv to index.html for" $file
      echo '<br><font style="background-color: #00FF00">>></font> Something went wrong with the mv to index.html for' $file '</font>' >> $Logfile
      sed -i '12i <p></p><center><font style="background-color: #00FF00"> 810 ERROR FOUND - Webmaster, check Logfile </font></center><p></p>' $out
      exit 1
  else
    echo
    echo "Finished!"
    echo
    echo "Generated file was saved to" $pout
    echo
  fi
fi   

  if [ ${#pout} -lt 10 ]
  then  
    highlight "       It appears that $pout was not created properly    "
    echo '<br><font style="background-color: #00FF00">>></font> It appears that' $pout 'was not created properly</font>'  >> $Logfile
    errorfound=1
    Logflag=1
  fi

# Remove any remaining testfi.log in case of previous aborted run
if [ -e testfi.log ]
then
rm testfi.log
fi

# Send failure to create index message to the log file
cwd=$PWD
if [ ! -f $pout ] || [ -f $out ]
then
  Logflag=1
  echo '<br><font style="background-color: #00FF00">>></font> Unable to create" $pout 'at' $cwd </font>' >> $Logfile
  highlight "unable to create new" $pout "at" $cwd
  sed -i '12i <p></p><center><font style="background-color: #00FF00"> 805 ERROR FOUND - Webmaster, check Logfile </font></center><p></p>' $out
  if [ -f $errorsound ]
  then
    echo aplay -q $errorsound  && play -q $errorsound -t alsa
  fi
exit 1
  echo
fi

if [ $errorfound -eq 1 ]
# Post message on webpage to check the error log
then
    sed -i '12i <p></p><center><font style="background-color: #00FF00"> ERROR FOUND - Webmaster, check Logfile </font></center><p></p>' $pout
fi


let "timesthrough++"

continue

# End of main loop
done


  if [ $Logflag -eq 1 ]
  then
      echo 1 >> ${flagdir}/.flagerrorfile
   fi

  if [ $NDflag -eq 1 ]
  then
      echo 1 >> ${flagdir}/.NDflagfile
  fi


# Next recursion (directory)



